@@ -15,7 +15,7 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: socket.c,v 1.30.18.20.12.5.6.2 2008/09/04 05:47:09 each Exp $ */
+/* $Id: socket.c,v 1.52.94.2.4.5.6.2 2008/09/05 00:29:15 each Exp $ */
 
 /* This code uses functions which are only available on Server 2003 and
  * higher, and Windows XP and higher.
@@ -208,15 +208,15 @@ enum {
  * Used value-result for recvmsg, value only for sendmsg.
  */
 struct msghdr {
-	SOCKADDR to_addr;		/* UDP send/recv address */
+	SOCKADDR_STORAGE to_addr;	/* UDP send/recv address */
 	int      to_addr_len;		/* length of the address */
-        WSABUF  *msg_iov;		/* scatter/gather array */
-        u_int   msg_iovlen;             /* # elements in msg_iov */
-        void	*msg_control;           /* ancillary data, see below */
-        u_int   msg_controllen;         /* ancillary data buffer len */
+	WSABUF  *msg_iov;		/* scatter/gather array */
+	u_int   msg_iovlen;             /* # elements in msg_iov */
+	void	*msg_control;           /* ancillary data, see below */
+	u_int   msg_controllen;         /* ancillary data buffer len */
 	int	msg_totallen;		/* total length of this message */
 } msghdr;
-	
+
 /*
  * The size to raise the receive buffer to.
  */
@@ -243,6 +243,8 @@ struct isc_socket {
 	unsigned int		references; /* EXTERNAL references */
 	SOCKET			fd;	/* file handle */
 	int			pf;	/* protocol family */
+	char			name[16];
+	void *			tag;
 
 	/*
 	 * Each recv() call uses this buffer.  It is a per-socket receive
@@ -252,7 +254,7 @@ struct isc_socket {
 	 * calls.  It also allows us to read-ahead in some cases.
 	 */
 	struct {
-		SOCKADDR	from_addr;	   // UDP send/recv address
+		SOCKADDR_STORAGE	from_addr;	   // UDP send/recv address
 		int		from_addr_len;	   // length of the address
 		char		*base;		   // the base of the buffer
 		char		*consume_position; // where to start copying data from next
@@ -271,8 +273,7 @@ struct isc_socket {
 				connected : 1,
 				pending_connect : 1, /* connect pending */
 				bound : 1;	/* bound to local addr */
-
-	unsigned int		pending_iocp;  /* Should equal the counters below. Debug. */
+	unsigned int		pending_iocp;	/* Should equal the counters below. Debug. */
 	unsigned int		pending_recv;  /* Number of outstanding recv() calls. */
 	unsigned int		pending_send;  /* Number of outstanding send() calls. */
 	unsigned int		pending_accept; /* Number of outstanding accept() calls. */
@@ -531,7 +532,7 @@ iocompletionport_init(isc_socketmgr_t *manager) {
 
 	/*
 	 * Worker threads for servicing the I/O
- 	 */
+	 */
 	iocompletionport_createthreads(manager->maxIOCPThreads, manager);
 }
 
@@ -662,7 +663,7 @@ initialise(void) {
 void
 InitSockets(void) {
 	RUNTIME_CHECK(isc_once_do(&initialise_once,
-                                  initialise) == ISC_R_SUCCESS);
+				  initialise) == ISC_R_SUCCESS);
 	if (!initialised)
 		exit(1);
 }
@@ -679,7 +680,7 @@ internal_sendmsg(isc_socket_t *sock, IoCompletionInfo *lpo,
 	*Error = 0;
 	Result = WSASendTo(sock->fd, messagehdr->msg_iov,
 			   messagehdr->msg_iovlen, &BytesSent,
-			   Flags, &messagehdr->to_addr,
+			   Flags, (SOCKADDR *)&messagehdr->to_addr,
 			   messagehdr->to_addr_len, (LPWSAOVERLAPPED)lpo,
 			   NULL);
 
@@ -753,7 +754,7 @@ queue_receive_request(isc_socket_t *sock) {
 	Error = 0;
 	Result = WSARecvFrom((SOCKET)sock->fd, iov, 1,
 			     &NumBytes, &Flags,
-			     &sock->recvbuf.from_addr,
+			     (SOCKADDR *)&sock->recvbuf.from_addr,
 			     &sock->recvbuf.from_addr_len,
 			     (LPWSAOVERLAPPED)lpo, NULL);
 
@@ -768,7 +769,7 @@ queue_receive_request(isc_socket_t *sock) {
 			break;
 
 		default:
-			isc_result = isc__errno2result(Result);
+			isc_result = isc__errno2result(Error);
 			if (isc_result == ISC_R_UNEXPECTED)
 				UNEXPECTED_ERROR(__FILE__, __LINE__,
 					"WSARecvFrom: Windows error code: %d, isc result %d",
@@ -1035,10 +1036,15 @@ static void
 set_dev_address(isc_sockaddr_t *address, isc_socket_t *sock,
 		isc_socketevent_t *dev)
 {
-	if (address != NULL)
-		dev->address = *address;
-	else
+	if (sock->type == isc_sockettype_udp) {
+		if (address != NULL)
+			dev->address = *address;
+		else
+			dev->address = sock->address;
+	} else if (sock->type == isc_sockettype_tcp) {
+		INSIST(address == NULL);
 		dev->address = sock->address;
+	}
 }
 
 static void
@@ -1454,6 +1460,7 @@ allocate_socket(isc_socketmgr_t *manager, isc_sockettype_t type,
 	sock->connected = 0;
 	sock->pending_connect = 0;
 	sock->bound = 0;
+	memset(sock->name, 0, sizeof(sock->name));	// zero the name field
 	_set_state(sock, SOCK_INITIALIZED);
 
 	sock->recvbuf.len = 65536;
@@ -1938,9 +1945,9 @@ internal_accept(isc_socket_t *sock, IoCompletionInfo *lpo, int accept_errno) {
 	isc_socket_newconnev_t *adev;
 	isc_result_t result = ISC_R_SUCCESS;
 	isc_socket_t *nsock;
-	struct sockaddr_in *localaddr;
+	struct sockaddr *localaddr;
 	int localaddr_len = sizeof(*localaddr);
-	struct sockaddr_in *remoteaddr;
+	struct sockaddr *remoteaddr;
 	int remoteaddr_len = sizeof(*remoteaddr);
 
 	INSIST(VALID_SOCKET(sock));
@@ -1978,7 +1985,7 @@ internal_accept(isc_socket_t *sock, IoCompletionInfo *lpo, int accept_errno) {
 	 * and return the new socket.
 	 */
 	ISCGetAcceptExSockaddrs(lpo->acceptbuffer, 0,
-		sizeof(SOCKADDR) + 16, sizeof(SOCKADDR) + 16,
+		sizeof(SOCKADDR_STORAGE) + 16, sizeof(SOCKADDR_STORAGE) + 16,
 		(LPSOCKADDR *)&localaddr, &localaddr_len,
 		(LPSOCKADDR *)&remoteaddr, &remoteaddr_len);
 	memcpy(&adev->address.type, remoteaddr, remoteaddr_len);
@@ -2987,7 +2994,7 @@ isc_socket_sendto2(isc_socket_t *sock, isc_region_t *region,
 
 isc_result_t
 isc_socket_bind(isc_socket_t *sock, isc_sockaddr_t *sockaddr,
-	        unsigned int options) {
+		unsigned int options) {
 	int bind_errno;
 	char strbuf[ISC_STRERRORSIZE];
 	int on = 1;
@@ -3194,7 +3201,7 @@ isc_socket_accept(isc_socket_t *sock,
 					    sizeof(IoCompletionInfo));
 	RUNTIME_CHECK(lpo != NULL);
 	lpo->acceptbuffer = (void *)HeapAlloc(hHeapHandle, HEAP_ZERO_MEMORY,
-		(sizeof(SOCKADDR) + 16) * 2);
+		(sizeof(SOCKADDR_STORAGE) + 16) * 2);
 	RUNTIME_CHECK(lpo->acceptbuffer != NULL);
 
 	lpo->adev = adev;
@@ -3204,8 +3211,8 @@ isc_socket_accept(isc_socket_t *sock,
 		    nsock->fd,				/* Accepted Socket */
 		    lpo->acceptbuffer,			/* Buffer for initial Recv */
 		    0,					/* Length of Buffer */
-		    sizeof(SOCKADDR) + 16,		/* Local address length + 16 */
-		    sizeof(SOCKADDR) + 16,		/* Remote address lengh + 16 */
+		    sizeof(SOCKADDR_STORAGE) + 16,		/* Local address length + 16 */
+		    sizeof(SOCKADDR_STORAGE) + 16,		/* Remote address lengh + 16 */
 		    (LPDWORD)&lpo->received_bytes,	/* Bytes Recved */
 		    (LPOVERLAPPED)lpo			/* Overlapped structure */
 		    );
@@ -3586,6 +3593,32 @@ isc_socket_permunix(isc_sockaddr_t *addr, isc_uint32_t perm,
 	return (ISC_R_NOTIMPLEMENTED);
 }
 
+void
+isc_socket_setname(isc_socket_t *socket, const char *name, void *tag) {
+
+	/*
+	 * Name 'socket'.
+	 */
+
+	REQUIRE(VALID_SOCKET(socket));
+
+	LOCK(&socket->lock);
+	memset(socket->name, 0, sizeof(socket->name));
+	strncpy(socket->name, name, sizeof(socket->name) - 1);
+	socket->tag = tag;
+	UNLOCK(&socket->lock);
+}
+
+const char *
+isc_socket_getname(isc_socket_t *socket) {
+	return (socket->name);
+}
+
+void *
+isc_socket_gettag(isc_socket_t *socket) {
+	return (socket->tag);
+}
+
 void
 isc__socketmgr_setreserved(isc_socketmgr_t *manager, isc_uint32_t reserved) {
 	UNUSED(manager);
